@@ -16,8 +16,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# $Id$
-
 """Configuration"""
 
 import gobject
@@ -30,7 +28,6 @@ from gettext import gettext as _
 import logging
 log = logging.getLogger("CONFIG")
 
-from cf import USER_CONF
 
 class Config(gobject.GObject):
     """Configuration object
@@ -61,28 +58,26 @@ class Config(gobject.GObject):
         specific data here.
 
 
-    Usage example
-    =============
+    :Usage example:
 
-        .. sourcecode:: python
+    .. sourcecode:: python
 
-            >>> app.config.get("foo.bar") # Not set yet, None is default
-            None
-            >>> app.config.set("foo.bar", True)
-            >>> app.config.get("foo.bar")
-            True
-            >>> app.config.set("foo.bar", ["Completely", "different"]) # No type check!
-            >>> print " ".join(app.config.get("foo.bar"))
-            Completely different
+       >>> app.config.get("foo.bar") # Not set yet, None is default
+       None
+       >>> app.config.set("foo.bar", True)
+       >>> app.config.get("foo.bar")
+       True
+       >>> app.config.set("foo.bar", ["Completely", "different"]) # No type check!
+       >>> print " ".join(app.config.get("foo.bar"))
+       Completely different
 
 
-    Signals
-    =======
+    :Signals:
 
-        changed
-            ``def callback(config, key, value, user_oaram1, ...)``
+    changed
+      ``def callback(config, key, value, user_oaram1, ...)``
 
-            Emitted when a option has changed.
+      Emitted when a option has changed.
 
 
     .. _CFApplication: cf.app.CFApplication.html
