@@ -16,24 +16,21 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# $Id$
-
 """Builtin plugins"""
 
+import logging
+
 from cf.filter.exportfilter import CSVExportFilter
 from cf.filter.exportfilter import OOCalcExportFilter
 
-try: from cf.backends.ldapbe import LDAPBackend
-except ImportError: pass
 try: from cf.backends.mysql import MySQLBackend
-except ImportError: pass
-try: from cf.backends.postgres import PostgresBackend
-except ImportError: pass
-try: from cf.backends.sqlite import SQLiteBackend
-except ImportError: pass
-try: from cf.backends.mssql import MsSQLBackend
-except ImportError: pass
+except ImportError, err:
+    logging.warning('Could not import MySQLBackend: %s', err)
+try: from cf.backends.oracle import OracleBackend
+except ImportError, err:
+    logging.warning('Could not import OracleBackend; %s', err)
 
 from cf.shell import CFShell
 from cf.library import SQLLibraryPlugin
-from cf.ui.refviewer import ReferenceViewer
+from cf.nativeshell import NativeShellPlugin
+
