@@ -16,15 +16,18 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# $Id$
-
 """Preferences"""
 
+from inspect import isclass
+
 import gtk
 import gobject
-import gnome
-import gconf
-import gnomevfs
+
+try:
+    import gconf
+    HAVE_GCONF = True
+except ImportError:
+    HAVE_GCONF = False
 
 import os
 import sys
@@ -34,24 +37,26 @@ from gettext import gettext as _
 import logging
 log = logging.getLogger("PREFS")
 
-from inspect import isclass
 
-try:
-    from xml.etree import ElementTree as etree
-except ImportError:
-    from lxml import etree
-from kiwi.ui import dialogs
 
 import cf
-from cf.ui import GladeWidget
-from cf.ui.widgets import ProgressDialog
+from cf.ui import dialogs
 from cf.plugins.core import GenericPlugin
 
-class PreferencesDialog(GladeWidget):
 
-    def __init__(self, app, mode="editor"):
-        GladeWidget.__init__(self, app, "crunchyfrog", "preferences_dialog")
-        self.sync_repo_file(silent=True)
+class PreferencesDialog(object):
+
+    def __init__(self, win, mode="editor"):
+        self.win = win
+        self.app = win.app
+        self.builder = gtk.Builder()
+        self.builder.set_translation_domain('crunchyfrog')
+        fname = self.app.get_glade_file('preferences.glade')
+        self.builder.add_from_file(fname)
+        self.builder.connect_signals(self)
+        self.dlg = self.builder.get_object('preferences_dialog')
+        self._setup_widget()
+        self._setup_connections()
         self.refresh()
         if mode == "editor":
             curr_page = 1
@@ -59,31 +64,33 @@ class PreferencesDialog(GladeWidget):
             curr_page = 2
         else:
             curr_page = 0
-        self.xml.get_widget("notebook1").set_current_page(curr_page)
+        self.builder.get_object("notebook1").set_current_page(curr_page)
 
     def _setup_widget(self):
         self._setup_plugins()
         self._setup_editor()
+        self._setup_shortcuts()
         # Fix secondary button
-        btn = self.xml.get_widget("btn_help")
-        box = self.xml.get_widget("dialog-action_area1")
+        btn = self.builder.get_object("btn_help")
+        box = self.builder.get_object("dialog-action_area1")
         box.set_child_secondary(btn, True)
 
     def _setup_editor(self):
         model = gtk.ListStore(int, str, gtk.gdk.Pixbuf)
-        iconview = self.xml.get_widget("editor_iconview")
+        iconview = self.builder.get_object("editor_iconview")
         iconview.set_model(model)
         iconview.set_text_column(1)
         iconview.set_pixbuf_column(2)
-        it = gtk.icon_theme_get_default()
         def get_pb(stock):
-            return it.load_icon(stock, 36, gtk.ICON_LOOKUP_FORCE_SVG)
+            return self.app.load_icon(stock, 36, gtk.ICON_LOOKUP_FORCE_SVG)
         model.append([0, _(u"General"), get_pb("gtk-execute")])
         model.append([1, _(u"View"), get_pb("gtk-justify-left")])
         model.append([2, _(u"Editor"), get_pb("gtk-edit")])
         model.append([3, _(u"Fonts & Colors"), get_pb("gtk-font")])
+        model.append([4, _(u'Keyboard\nShortcuts'),
+                      get_pb('preferences-desktop-keyboard-shortcuts')])
         iconview.connect("selection-changed", self.on_editor_selection_changed)
-        schemes = self.xml.get_widget("editor_schemes")
+        schemes = self.builder.get_object("editor_schemes")
         model = gtk.ListStore(str, str)
         model.set_sort_column_id(1, gtk.SORT_ASCENDING)
         schemes.set_model(model)
@@ -116,7 +123,7 @@ class PreferencesDialog(GladeWidget):
                                           bool, # 4 active visible
                                           )
         self.plugin_model.set_sort_column_id(2, gtk.SORT_ASCENDING)
-        self.plugin_list = self.xml.get_widget("plugin_list")
+        self.plugin_list = self.builder.get_object("plugin_list")
         self.plugin_list.set_model(self.plugin_model)
         # label
         col = gtk.TreeViewColumn()
@@ -135,12 +142,57 @@ class PreferencesDialog(GladeWidget):
         sel = self.plugin_list.get_selection()
         sel.connect("changed", self.on_plugin_selection_changed)
 
+    def _setup_shortcuts(self):
+        self.shortcuts_model = gtk.TreeStore(str,                  # 0 label
+                                             int,                  # 1 keyval
+                                             gtk.gdk.ModifierType, # 2 mods
+                                             bool,                 # 3 visible
+                                             str,                  # 4 tooltip
+                                             gobject.TYPE_PYOBJECT,# 5 action
+                                             )
+        self.list_shortcuts = self.builder.get_object('list_shortcuts')
+        col = gtk.TreeViewColumn(_('Action'), gtk.CellRendererText(), text=0)
+        self.list_shortcuts.append_column(col)
+        renderer = gtk.CellRendererAccel()
+        renderer.connect('accel-edited', self.on_accel_edited)
+        col = gtk.TreeViewColumn(_(u'Shortcut'), renderer,
+                                 accel_key=1, accel_mods=2, visible=3,
+                                 editable=3)
+        self.list_shortcuts.append_column(col)
+        self.shortcuts_model.set_sort_column_id(0, gtk.SORT_ASCENDING)
+        self.list_shortcuts.set_model(self.shortcuts_model)
+
     def _setup_connections(self):
         self.app.plugins.connect("plugin-added", self.on_plugin_added)
         self.app.plugins.connect("plugin-removed", self.on_plugin_removed)
 
-    def on_editor_open_in_window_toggled(self, toggle):
-        self.app.config.set("editor.open_in_window", toggle.get_active())
+    def run(self):
+        """Run the dialog."""
+        return self.dlg.run()
+
+    def destroy(self):
+        """Destroy the dialog."""
+        self.dlg.destroy()
+
+    def on_accel_edited(self, renderer, path, accel_key, accel_mods,
+                        hardware_keycode):
+        model = self.shortcuts_model
+        iter_ = model.get_iter(path)
+        action = model.get_value(iter_, 5)
+        if not action:
+            return
+        if accel_key == gtk.keysyms.Delete and not accel_mods:
+            accel_key = accel_mods = 0
+        model.set_value(iter_, 1, accel_key)
+        model.set_value(iter_, 2, accel_mods)
+        gtk.accel_map_change_entry(action.get_accel_path(),
+                                   accel_key, accel_mods, True)
+
+    def on_editor_reuse_conn_toggled(self, toggle):
+        self.app.config.set("editor.reuse_connection", toggle.get_active())
+
+    def on_editor_hide_results_toggled(self, toggle):
+        self.app.config.set('editor.hide_results_pane', toggle.get_active())
 
     def on_editor_option_changed(self, widget, *args):
         option = widget.get_data("config_option")
@@ -155,11 +207,11 @@ class PreferencesDialog(GladeWidget):
             model, iter = widget.get_selected()
             conf.set(option, model.get_value(iter, 0))
         if option == "editor.wrap_text":
-            self.xml.get_widget("editor_wrap_split").set_sensitive(widget.get_active())
+            self.builder.get_object("editor_wrap_split").set_sensitive(widget.get_active())
         if option == "editor.right_margin":
-            self.xml.get_widget("editor_right_margin_position_box").set_sensitive(widget.get_active())
+            self.builder.get_object("editor_right_margin_position_box").set_sensitive(widget.get_active())
         if option == "editor.default_font":
-            self.xml.get_widget("editor_font_box").set_sensitive(not widget.get_active())
+            self.builder.get_object("editor_font_box").set_sensitive(not widget.get_active())
         if option == "plugins.repo_enabled":
             gobject.idle_add(self.refresh_plugins)
 
@@ -167,52 +219,21 @@ class PreferencesDialog(GladeWidget):
         model = iconview.get_model()
         for path in iconview.get_selected_items():
             iter = model.get_iter(path)
-            nb = self.xml.get_widget("editor_notebook")
+            nb = self.builder.get_object("editor_notebook")
             nb.set_current_page(model.get_value(iter, 0))
 
+    def on_help(self, *args):
+        self.app.show_help()
+
     def on_plugin_active_toggled(self, renderer, path):
         iter = self.plugin_model.get_iter(path)
         plugin = self.plugin_model.get_value(iter, 0)
-        if isinstance(plugin, etree._Element):
-            self._plugin_download_and_activate(plugin)
-        elif issubclass(plugin, GenericPlugin):
+        if issubclass(plugin, GenericPlugin):
             self.plugin_model.set_value(iter, 1, not renderer.get_active())
             gobject.idle_add(self.app.plugins.set_active, plugin, not renderer.get_active())
 
-    def _plugin_download_and_activate(self, plugin):
-        if not dialogs.yesno(_(u"Download plugin %r?") % plugin.xpath("//*/name")[0].text) == gtk.RESPONSE_YES:
-            return
-        def progress_cb(info, dlg):
-            if info.bytes_total:
-                fraction = info.total_bytes_copied/float(info.bytes_total)
-            else:
-                fraction = 0
-            dlg.set_progress(fraction)
-            return True
-        url = os.path.join(self.app.config.get("plugins.repo_url"), plugin.xpath("//*/egg")[0].text)
-        source = gnomevfs.URI(url)
-        dest = gnomevfs.URI(os.path.join(cf.USER_PLUGIN_URI, plugin.xpath("//*/egg")[0].text))
-        try:
-            dlg = ProgressDialog(self.app)
-            dlg.show_all()
-            dlg.set_info(_(u"Downloading plugin %r...") % plugin.xpath("//*/name")[0].text)
-            gnomevfs.xfer_uri(source, dest, gnomevfs.XFER_DEFAULT,
-                              gnomevfs.XFER_ERROR_MODE_QUERY,
-                              gnomevfs.XFER_OVERWRITE_MODE_QUERY,
-                              progress_cb, dlg)
-            dlg.set_info(_(u"Plugin downloaded."))
-            self.app.plugins.refresh()
-            self.refresh_plugins_repo()
-        except:
-            err = sys.exc_info()[1]
-            log.error("Plugin download failed: %s" % err)
-            log.error("URL: %s" % url)
-            dlg.set_error(_(u"Failed to install plugin %r") % plugin.xpath("//*/name")[0].text)
-        dlg.set_finished(True)
-
     def on_plugin_added(self, manager, plugin):
         iter = self.plugin_model.get_iter_first()
-        it = gtk.icon_theme_get_default()
         while iter:
             if self.plugin_model.get_value(iter, 0) == plugin.plugin_type:
                 break
@@ -222,7 +243,9 @@ class PreferencesDialog(GladeWidget):
             if plugin.description:
                 lbl += "\n"+plugin.description
             if plugin.icon:
-                ico = it.load_icon(plugin.icon, gtk.ICON_SIZE_LARGE_TOOLBAR, gtk.ICON_LOOKUP_FORCE_SVG)
+                ico = self.app.load_icon(plugin.icon,
+                                         gtk.ICON_SIZE_LARGE_TOOLBAR,
+                                         gtk.ICON_LOOKUP_FORCE_SVG)
             else:
                 ico = None
             citer = self.plugin_model.append(iter)
@@ -232,7 +255,6 @@ class PreferencesDialog(GladeWidget):
                       2, lbl,
                       3, ico,
                       4, True)
-        self.refresh_plugins_repo()
 
     def on_plugin_install(self, *args):
         dlg = gtk.FileChooserDialog(_(u"Install plugin"), None,
@@ -260,7 +282,6 @@ class PreferencesDialog(GladeWidget):
                 while citer:
                     if self.plugin_model.get_value(citer, 0) == plugin:
                         self.plugin_model.remove(citer)
-                        self.refresh_plugins_repo()
                         return
                     citer = self.plugin_model.iter_next(citer)
             iter = self.plugin_model.iter_next(iter)
@@ -300,11 +321,11 @@ class PreferencesDialog(GladeWidget):
             return
         obj = model.get_value(iter, 0)
         if isclass(obj) and issubclass(obj, GenericPlugin):
-            self.xml.get_widget("plugin_about").set_sensitive(True)
-            self.xml.get_widget("plugin_prefs").set_sensitive(obj.has_custom_options)
+            self.builder.get_object("plugin_about").set_sensitive(True)
+            self.builder.get_object("plugin_prefs").set_sensitive(obj.has_custom_options)
         else:
-            self.xml.get_widget("plugin_about").set_sensitive(False)
-            self.xml.get_widget("plugin_prefs").set_sensitive(False)
+            self.builder.get_object("plugin_about").set_sensitive(False)
+            self.builder.get_object("plugin_prefs").set_sensitive(False)
 
     def on_plugin_sync_repo(self, *args):
         self.sync_repo_file()
@@ -317,15 +338,25 @@ class PreferencesDialog(GladeWidget):
     def refresh(self):
         self.refresh_editor()
         self.refresh_plugins()
+        self.refresh_shortcuts()
 
     def refresh_editor(self):
         config = self.app.config
-        gw = self.xml.get_widget
-        gw("editor_open_separate").set_active(config.get("editor.open_in_window"))
+        gw = self.builder.get_object
+        gw('editor_reuse_connection').set_data('config_option',
+                                               'editor.reuse_connection')
+        gw("editor_reuse_connection").set_active(
+            config.get("editor.reuse_connection"))
+        gw('editor_hide_results').set_data('config_option',
+                                           'editor.hide_results_pane')
+        gw('editor_hide_results').set_active(
+            config.get('editor.hide_results_pane'))
         gw("editor_replace_variables").set_data("config_option", "editor.replace_variables")
         gw("editor_replace_variables").set_active(config.get("editor.replace_variables"))
         gw("sqlparse_enabled").set_data("config_option", "sqlparse.enabled")
         gw("sqlparse_enabled").set_active(config.get("sqlparse.enabled"))
+        gw('autocompl_tab').set_data('config_option', 'editor.tabcompletion')
+        gw('autocompl_tab').set_active(config.get('editor.tabcompletion'))
         gw("editor_wrap_text").set_data("config_option", "editor.wrap_text")
         gw("editor_wrap_text").set_active(config.get("editor.wrap_text"))
         gw("editor_wrap_split").set_data("config_option", "editor.wrap_split")
@@ -350,8 +381,11 @@ class PreferencesDialog(GladeWidget):
         gw("editor_auto_indent").set_active(config.get("editor.auto_indent"))
         gw("editor_default_font").set_data("config_option", "editor.default_font")
         gw("editor_default_font").set_active(config.get("editor.default_font"))
-        client = gconf.client_get_default()
-        default_font = client.get_string("/desktop/gnome/interface/monospace_font_name")
+        if HAVE_GCONF:
+            client = gconf.client_get_default()
+            default_font = client.get_string("/desktop/gnome/interface/monospace_font_name")
+        else:
+            default_font = 'Monospace 10'
         gw("editor_default_font").set_label(gw("editor_default_font").get_label() % default_font)
         gw("editor_font_box").set_sensitive(not config.get("editor.default_font"))
         gw("editor_font").set_data("config_option", "editor.font")
@@ -359,12 +393,11 @@ class PreferencesDialog(GladeWidget):
 
     def refresh_plugins(self):
         # Repo
-        self.xml.get_widget("plugin_enable_repo").set_data("config_option", "plugins.repo_enabled")
-        self.xml.get_widget("plugin_enable_repo").set_active(self.app.config.get("plugins.repo_enabled"))
+        self.builder.get_object("plugin_enable_repo").set_data("config_option", "plugins.repo_enabled")
+        self.builder.get_object("plugin_enable_repo").set_active(self.app.config.get("plugins.repo_enabled"))
         # Plugins
         model = self.plugin_model
         model.clear()
-        it = gtk.icon_theme_get_default()
         for key, value in self.app.plugins.plugin_types.items():
             iter = model.append(None)
             model.set(iter,
@@ -381,7 +414,9 @@ class PreferencesDialog(GladeWidget):
                     lbl += '\n<span color="red">'
                     lbl += _(u'ERROR')+': '+plugin.INIT_ERROR+'</span>'
                 if plugin.icon:
-                    ico = it.load_icon(plugin.icon, gtk.ICON_SIZE_LARGE_TOOLBAR, gtk.ICON_LOOKUP_FORCE_SVG)
+                    ico = self.app.load_icon(plugin.icon,
+                                             gtk.ICON_SIZE_LARGE_TOOLBAR,
+                                             gtk.ICON_LOOKUP_FORCE_SVG)
                 else:
                     ico = None
                 citer = model.append(iter)
@@ -391,7 +426,6 @@ class PreferencesDialog(GladeWidget):
                           2, lbl,
                           3, ico,
                           4, not bool(plugin.INIT_ERROR))
-        gobject.idle_add(self.refresh_plugins_repo)
 
     def _plugin_iter_for_ep(self, ep_name):
         model = self.plugin_list.get_model()
@@ -402,99 +436,36 @@ class PreferencesDialog(GladeWidget):
             iter = model.iter_next(iter)
         return model, None
 
-    def refresh_plugins_repo(self):
-        """Refresh repository plugins"""
-        model = self.plugin_list.get_model()
-        iter = model.get_iter_first()
-        while iter:
-            citer = model.iter_children(iter)
-            while citer:
-                obj = model.get_value(citer, 0)
-                if isinstance(obj, etree._Element):
-                    model.remove(citer)
-                citer = model.iter_next(citer)
-            iter = model.iter_next(iter)
-        if not self.app.config.get("plugins.repo_enabled"):
-            return
-        if not os.path.isfile(cf.USER_PLUGIN_REPO):
-            if not self.sync_repo_file():
-                return
-        dom = etree.parse(cf.USER_PLUGIN_REPO)
-        for plugin in dom.xpath("//*/plugin"):
-            ptype = plugin.get("type")
-            model, iter = self._plugin_iter_for_ep(int(ptype))
-            if not iter:
-                log.error("Invalid plugin type %r" % ptype)
+    def refresh_shortcuts(self):
+        model = self.shortcuts_model
+        model.clear()
+        for group in self.win.ui.get_action_groups():
+            if group.get_name() == 'clipboard':
                 continue
-            citer = model.iter_children(iter)
-            skip = False
-            while citer:
-                x = model.get_value(citer, 0)
-                if isclass(x) and issubclass(x, GenericPlugin) \
-                and x.id == plugin.get("id"):
-                    skip = True
-                    break
-                elif isinstance(x, etree._Element) \
-                and x.get("id") == plugin.get("id"):
-                    skip = True
-                    break
-                citer = model.iter_next(citer)
-            if skip:
-                continue
-            lbl = '<b>[REPO] %s</b>' % plugin.xpath("//*/name")[0].text or _(u"Unknown")
-            if plugin.xpath("//*/description"):
-                lbl += "\n"+plugin.xpath("//*/description")[0].text
-            it = gtk.icon_theme_get_default()
-            ico = it.load_icon("stock_internet", gtk.ICON_SIZE_LARGE_TOOLBAR, gtk.ICON_LOOKUP_FORCE_SVG)
-            citer = model.append(iter)
-            model.set(citer,
-                      0, plugin,
-                      1, False,
-                      2, lbl,
-                      3, ico,
-                      4, True)
-
-    def sync_repo_file(self, silent=False):
-        """Synchronize plugin repo.xml"""
-        def progress_cb(info, dlg):
-            if info.bytes_total:
-                fraction = info.total_bytes_copied/float(info.bytes_total)
-            else:
-                fraction = 0
-            dlg.set_progress(fraction)
-            return True
-        # XXX currently disabled, should be replaced by a Capuchin backend
-        return
-        url = os.path.join(self.app.config.get("plugins.repo_url"), "repo.xml")
-        source = gnomevfs.URI(url)
-        dest = gnomevfs.URI(cf.USER_PLUGIN_REPO_URI)
-        try:
-            if not silent:
-                dlg = ProgressDialog(self.app)
-                dlg.show_all()
-                dlg.set_info(_(u"Downloading repository data..."))
-                gnomevfs.xfer_uri(source, dest, gnomevfs.XFER_DEFAULT,
-                                  gnomevfs.XFER_ERROR_MODE_QUERY,
-                                  gnomevfs.XFER_OVERWRITE_MODE_QUERY,
-                                  progress_cb, dlg)
-                dlg.set_info(_(u"Repository data synchronized."))
-                self.refresh_plugins_repo()
-            else:
-                def fake_cb(*args):
-                    return True
-                gnomevfs.xfer_uri(source, dest, gnomevfs.XFER_DEFAULT,
-                                  gnomevfs.XFER_ERROR_MODE_QUERY,
-                                  gnomevfs.XFER_OVERWRITE_MODE_QUERY,
-                                  fake_cb)
-            retval = True
-        except:
-            err = sys.exc_info()[1]
-            log.error("Plugin synchro failed: %s" % err)
-            log.error("URL: %s" % url)
-            if not silent:
-                dlg.set_error(_(u"Failed to load repository data"))
-            self.xml.get_widget("plugin_enable_repo").set_active(False)
-            retval = False
-        if not silent:
-            dlg.set_finished(True)
-        return retval
+            iter_ = model.append(None)
+            lbl = group.get_data('cf::label') or group.get_name()
+            model.set(iter_, 0, lbl, 3, False)
+            for action in group.list_actions():
+                # Don't display menu items with submenus
+                if action.get_name().endswith('menu-action') \
+                or action.get_name().startswith('activate-editor'):
+                    continue
+                citer = model.append(iter_)
+                accel_path = action.get_accel_path()
+                if accel_path is None:
+                    keyval = mods = None
+                else:
+                    shortcut = gtk.accel_map_lookup_entry(accel_path)
+                    if shortcut is not None:
+                        keyval, mods = shortcut
+                    else:
+                        keyval = mods = None
+                model.set(citer,
+                          0, action.props.label.replace('_', ''),
+                          3, True,
+                          4, action.props.tooltip,
+                          5, action)
+                if keyval is not None:
+                    model.set(citer, 1, keyval)
+                if mods is not None:
+                    model.set(citer, 2, mods)
