@@ -16,36 +16,32 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# $Id$
-
 """Common widgets"""
 
 from gettext import gettext as _
 import os
 
-import gnomevfs
 import gobject
 import gtk
 import pango
 
-from kiwi.ui import dialogs
-
-from cf.backends import DBConnectError, DBConnection
-from cf.datasources import DatasourceInfo
-from cf.ui import GladeWidget
+from cf.db import Connection, Datasource
 
 
-class ConnectionButton(GladeWidget):
+class ConnectionButton(gtk.MenuToolButton):
     """Connection chooser used in toolbars.
 
     It is always bound to an SQLEditor.
     """
 
-    def __init__(self, app, xml):
-        GladeWidget.__init__(self, app, xml, "tb_connection")
+    def __init__(self, mainwin):
+        gtk.MenuToolButton.__init__(self, gtk.STOCK_CONNECT)
+        self.app = mainwin.app
+        self.mainwin = mainwin
+        self._editor = None
         self._setup_widget()
         self.set_editor(None)
-        self.app.datasources.connect("datasource-modified",
+        self.app.datasources.connect("datasource-changed",
                                      lambda *a: self.rebuild_menu())
         self.app.datasources.connect("datasource-added",
                                      lambda *a: self.rebuild_menu())
@@ -63,67 +59,9 @@ class ConnectionButton(GladeWidget):
         self._menu = gtk.Menu()
         self.set_menu(self._menu)
 
-    def on_new_connection(self, item, datasource_info):
-        try:
-            conn = datasource_info.dbconnect()
-            self._editor.set_connection(conn)
-            self.set_editor(self._editor)
-        except DBConnectError, err:
-            dialogs.error(_(u"Connection failed"), str(err))
-
-    def on_manage_connections(self, *args):
-        self.manage_connections()
-
-    def on_set_connection(self, item, connection):
-        self._editor.set_connection(connection)
-        self.set_editor(self._editor)
-
-    def manage_connections(self):
-        """Displays a dialog to manage connections."""
-        dlg = ConnectionsDialog(self.app)
-        dlg.run()
-        dlg.destroy()
-
     def rebuild_menu(self):
         """Rebuilds the drop-down menu."""
-        while self._menu.get_children():
-            self._menu.remove(self._menu.get_children()[0])
-        if not self._editor:
-            return
-        ghas_connections = False
-        dinfos = self.app.datasources.get_all()
-        dinfos.sort(lambda x, y: cmp(x.get_label().lower(),
-                                     y.get_label().lower()))
-        for datasource_info in dinfos:
-            item = gtk.MenuItem(datasource_info.get_label())
-            item.show()
-            self._menu.append(item)
-            menu = gtk.Menu()
-            has_connections = False
-            for conn in datasource_info.get_connections():
-                yitem = gtk.MenuItem((_(u"Connection")+" #%s"
-                                      % conn.conn_number))
-                yitem.connect("activate", self.on_set_connection, conn)
-                yitem.show()
-                menu.append(yitem)
-                has_connections = True
-                ghas_connections = True
-            if has_connections:
-                sep = gtk.SeparatorMenuItem()
-                sep.show()
-                menu.append(sep)
-            xitem = gtk.MenuItem(_(u"New connection"))
-            xitem.connect("activate", self.on_new_connection, datasource_info)
-            xitem.show()
-            menu.append(xitem)
-            item.set_submenu(menu)
-        sep = gtk.SeparatorMenuItem()
-        sep.show()
-        self._menu.append(sep)
-        item = gtk.MenuItem(_(u"Show connections"))
-        item.connect("activate", lambda *a: self.manage_connections())
-        item.show()
-        self._menu.append(item)
+        rebuild_connection_menu(self._menu, self.mainwin, self._editor)
 
     def set_editor(self, editor):
         """Associates an editor.
@@ -131,47 +69,109 @@ class ConnectionButton(GladeWidget):
         Arguments:
             editor: SQLEditor instance.
         """
+        from cf.ui.editor import Editor  # Import hook
+        if self._editor:
+            sig = self._editor.get_data('cf::connbtn::sig_conn_changed')
+            if sig:
+                self._editor.disconnect(sig)
+                self._editor.set_data('cf::connbtn::sig_conn_changed', None)
         self._editor = editor
         self.set_sensitive(bool(editor))
         self.rebuild_menu()
+        if editor is not None and isinstance(editor, Editor):
+            sig = editor.connect('connection-changed',
+                                 lambda *a: self.set_editor(editor))
+            editor.set_data('cf::connbtn::sig_conn_changed', sig)
         if editor and editor.connection:
             self._label.set_text(editor.connection.get_label())
             markup = ("<b>%s</b>\n%s #%s"
-                      % (editor.connection.datasource_info.get_label(),
-                         _(u"Connection"), editor.connection.conn_number))
+                      % (editor.connection.datasource.get_label(),
+                         _(u"Connection"), editor.connection.num))
             self.set_tooltip_markup(markup)
         else:
             self._label.set_text("<"+_(u"Not connected")+">")
             self.set_tooltip_markup(_(u"Click to open a connection"))
 
 
+def rebuild_connection_menu(menu, win, editor=None):
+    """Rebuilds the connection chooser menu.
+
+    Arguments:
+      menu: The menu to rebuild.
+      win: A main window instance.
+      editor: The editor for which the menu is build (default: None).
+    """
+    while menu.get_children():
+        menu.remove(menu.get_children()[0])
+    if editor is None:
+        return
+
+    def create_and_assign(menuitem, datasource, editor):
+        conn = datasource.dbconnect()
+        if conn is not None:
+            editor.set_connection(conn)
+
+    add_sep = False
+    for conn in win.app.datasources.get_connections():
+        add_sep = True
+        item = gtk.MenuItem(conn.get_label())
+        item.connect('activate', lambda m, c, e: e.set_connection(c),
+                     conn, editor)
+        item.show()
+        menu.append(item)
+
+    if add_sep:
+        item = gtk.SeparatorMenuItem()
+        item.show()
+        menu.append(item)
+
+    item = gtk.MenuItem(_(u'_New connection'))
+    item.show()
+    menu.append(item)
+
+    dsmenu = gtk.Menu()
+    dsinfos = win.app.datasources.get_all()
+    dsinfos.sort(lambda x, y: cmp(x.get_label().lower(),
+                                  y.get_label().lower()))
+    for dsinfo in dsinfos:
+        dsitem = gtk.MenuItem(dsinfo.get_label())
+        dsitem.show()
+        dsitem.connect('activate', create_and_assign, dsinfo, editor)
+        dsmenu.append(dsitem)
+    item.set_submenu(dsmenu)
+
+    action = win._get_action('query-show-connections')
+    item = action.create_menu_item()
+    item.show()
+    menu.append(item)
+    action = win._get_action('query-connection-disconnect')
+    item = action.create_menu_item()
+    action.set_sensitive(editor.connection is not None)
+    item.show()
+    menu.append(item)
+
+
 class DataExportDialog(gtk.FileChooserDialog):
-    """Export dialog.
+    """Modified gtk.FileChooserDialog for exporting data.
 
-    A modified gtk.FileChooserDialog for exporting data.
-
-    Usage example
-    =============
-
-        .. sourcecode:: python
-
-            >>> from cf.ui.widgets import DataExportDialog
-            >>> import gtk
-            >>> data = [["foo", 1, True], ["bar", 2, False]]
-            >>> selected = [2,]
-            >>> statement = ("select name, anumber, anotherfield "
-                              from foo limit 3;"
-            >>> description = (("name", str, None, None, None, None, None),
-            ...          ("anumber", int, None, None, None, None, None),
-            ...          ("anotherfield", bool, None, None, None, None, None))
-            >>> dlg = DataExportDialog(app, instance.widget,
-            ...                        data, selected, statement, description)
-            >>> if dlg.run() == gtk.RESPONSE_OK:
-            ...     dlg.hide()
-            ...     dlg.export_data()
-            ...
-            >>> dlg.destroy()
+    Usage example:
 
+      >>> from cf.ui.widgets import DataExportDialog
+      >>> import gtk
+      >>> data = [['foo', 1, True], ['bar', 2, False]]
+      >>> selected = [2,]
+      >>> statement = ('select name, anumber, anotherfield '
+          from foo limit 3;'
+      >>> description = (('name', str, None, None, None, None, None),
+      ...          ('anumber', int, None, None, None, None, None),
+      ...          ('anotherfield', bool, None, None, None, None, None))
+      >>> dlg = DataExportDialog(app, instance.widget,
+      ...                        data, selected, statement, description)
+      >>> if dlg.run() == gtk.RESPONSE_OK:
+      ...     dlg.hide()
+      ...     dlg.export_data()
+      ...
+      >>> dlg.destroy()
     """
 
     def __init__(self, app, parent, data, selected, statement, description):
@@ -228,8 +228,8 @@ class DataExportDialog(gtk.FileChooserDialog):
         self.set_extra_widget(vbox)
         self.edit_export_options = gtk.CheckButton(_(u"_Edit export options"))
         vbox.pack_start(self.edit_export_options, False, False)
-        self.export_selection = gtk.CheckButton(_(u("_Only export "
-                                                    "selected rows")))
+        self.export_selection = gtk.CheckButton(_((u"_Only export "
+                                                   u"selected rows")))
         self.export_selection.set_sensitive(bool(self.selected))
         vbox.pack_start(self.export_selection, False, False)
         self.open_file = gtk.CheckButton(_(u"_Open file when finished"))
@@ -295,101 +295,47 @@ class DataExportDialog(gtk.FileChooserDialog):
         and self.edit_export_options.get_active():
             opts.update(plugin.show_options(self.description, rows))
         plugin.export(self.description, rows, opts)
-        if self.open_file.get_active():
-            mime = gnomevfs.get_mime_type(opts["uri"])
-            app_desc = gnomevfs.mime_get_default_application(mime)
-            cmd = app_desc[2].split(" ")
-            os.spawnvp(os.P_NOWAIT, cmd[0], cmd+[opts["uri"]])
+        if self.open_file is not None and self.open_file.get_active():
+            gtk.show_uri(gtk.gdk.screen_get_default(), opts['uri'], 0)
 
 
-class ProgressDialog(GladeWidget):
-    """Progress dialog with a message
-
-    A simple window with a message, an icon and a progress bar.
-
-    Usage example
-    =============
-
-        .. sourcecode:: python
-
-            >>> from cf.ui.widgets import ProgressDialog
-            >>> dlg = ProgressDialog(app)
-            >>> dlg.show_all()
-            >>> dlg = ProgressDialog(app)
-            >>> dlg.set_modal(False) # only needed to run it in CF's shell
-            >>> dlg.show_all()
-            >>> dlg.set_info("This is an information")
-            >>> dlg.set_error("Uuups... an error occured")
-            >>> dlg.set_progress(0.75)
-            >>> dlg.set_finished(True)
-
-    """
-
-    def __init__(self, app, parent=None):
-        GladeWidget.__init__(self, app, "crunchyfrog", "progressdialog")
-        if parent:
-            self.reparent(parent)
-
-    def on_close(self, *args):
-        self.destroy()
-
-    def set_progress(self, fraction):
-        """Sets fraction for progress bar
-
-        :Parameter:
-            fraction
-                The fraction (``float``)
-        """
-        self.xml.get_widget("progress_progress").set_fraction(fraction)
-
-    def set_error(self, message):
-        """Displays an error
-
-        :Parameter:
-            message
-                Error message
-        """
-        message = gobject.markup_escape_text(message)
-        self.xml.get_widget("progress_label").set_markup("<b>"+message+"</b>")
-        self.xml.get_widget("progress_image").set_from_stock(
-            "gtk-dialog-error", gtk.ICON_SIZE_DIALOG)
-
-    def set_info(self, message):
-        """Displays an information
-
-        :Parameter:
-            message
-                A message
-        """
-        message = gobject.markup_escape_text(message)
-        self.xml.get_widget("progress_label").set_markup("<b>"+message+"</b>")
-        self.xml.get_widget("progress_image").set_from_stock(
-            "gtk-dialog-info", gtk.ICON_SIZE_DIALOG)
-
-    def set_finished(self, finished):
-        """Activates/deactivates close button
-
-        :Parameter:
-            finished
-                If ``True`` the close button gets sensitive.
-        """
-        self.xml.get_widget("progress_btn_close").set_sensitive(finished)
-
-
-class ConnectionsWidget(GladeWidget):
+class ConnectionsDialog(object):
     """Lists datasources and active connections"""
 
-    def __init__(self, app, xml="crunchyfrog"):
-        GladeWidget.__init__(self, app, xml, "connections_widget")
+    def __init__(self, win, xml="connectionsdialog"):
+        self.app = win.app
+        self.instance = win
+        self.builder = gtk.Builder()
+        self.builder.set_translation_domain('crunchyfrog')
+        self.builder.add_from_file(self.app.get_glade_file('%s.glade' % xml))
+        self.builder.connect_signals(self)
+        self.dlg = self.builder.get_object('connectionsdialog')
+        self._setup_widget()
+        self._setup_connections()
         self.refresh()
 
     def _setup_widget(self):
-        self.list_conn = self.xml.get_widget("list_connections")
-        model = gtk.TreeStore(gobject.TYPE_PYOBJECT, str)
-        model.set_sort_column_id(1, gtk.SORT_ASCENDING)
+        self.list_conn = self.builder.get_object("list_connections")
+        model = gtk.ListStore(gobject.TYPE_PYOBJECT,  # 0 object
+                              str,                    # 1 label
+                              bool,                   # 2 is separator
+                              int,                    # 3 font weight
+                              str,                    # 4 stock-id
+                              str,                    # 5 sort name
+                              )
+        model.set_sort_column_id(5, gtk.SORT_ASCENDING)
         self.list_conn.set_model(model)
-        col = gtk.TreeViewColumn("", gtk.CellRendererText(), text=1)
+        col = gtk.TreeViewColumn()
+        renderer = gtk.CellRendererPixbuf()
+        col.pack_start(renderer, expand=False)
+        col.add_attribute(renderer, 'stock-id', 4)
+        renderer = gtk.CellRendererText()
+        col.pack_start(renderer, expand=True)
+        col.add_attribute(renderer, 'text', 1)
+        col.add_attribute(renderer, 'weight', 3)
         self.list_conn.append_column(col)
+        self.list_conn.set_hover_expand(True)
+        self.list_conn.set_row_separator_func(lambda m, i: m.get_value(i, 2))
 
     def _setup_connections(self):
         sel = self.list_conn.get_selection()
@@ -398,26 +344,44 @@ class ConnectionsWidget(GladeWidget):
                                      self.on_datasource_added)
         self.app.datasources.connect("datasource-deleted",
                                      self.on_datasource_deleted)
-        self.app.datasources.connect("datasource-modified",
+        self.app.datasources.connect("datasource-changed",
                                      self.on_datasource_modified)
 
+    # ---------
+    # Callbacks
+    # ---------
+
+    def on_assign_to_editor(self, *args):
+        sel = self.list_conn.get_selection()
+        model, iter = sel.get_selected()
+        if not iter:
+            return
+        obj = model.get_value(iter, 0)
+        editor = self.instance.get_active_editor()
+        if editor is None:
+            return
+        editor.set_connection(obj)
+
     def on_connect(self, *args):
         sel = self.list_conn.get_selection()
         model, iter = sel.get_selected()
         if not iter:
             return
         obj = model.get_value(iter, 0)
-        if not isinstance(obj, DatasourceInfo):
+        if not isinstance(obj, Datasource):
             return
         conn = obj.dbconnect()
         conn.connect("closed", self.on_connection_closed)
         iter = model.get_iter_first()
         while iter:
-            if model.get_value(iter, 0) == conn.datasource_info:
-                citer = model.append(iter)
-                model.set(citer, 0, conn, 1, conn.get_label())
+            if model.get_value(iter, 0) == conn.datasource:
+                citer = model.insert_after(iter)
+                model.set(citer, 0, conn, 1, conn.get_label(short=True),
+                          5, conn.get_label())
                 break
             iter = model.iter_next(iter)
+        self.list_conn.expand_to_path(model.get_path(citer))
+        sel.select_iter(citer)
 
     def on_disconnect(self, *args):
         sel = self.list_conn.get_selection()
@@ -425,7 +389,7 @@ class ConnectionsWidget(GladeWidget):
         if not iter:
             return
         obj = model.get_value(iter, 0)
-        if not isinstance(obj, DBConnection):
+        if not isinstance(obj, Connection):
             return
         model.remove(iter)
         obj.close()
@@ -436,7 +400,7 @@ class ConnectionsWidget(GladeWidget):
             return
         iter = model.get_iter_first()
         while iter:
-            if model.get_value(iter, 0) == connection.datasource_info:
+            if model.get_value(iter, 0) == connection.datasource:
                 citer = model.iter_children(iter)
                 while citer:
                     if model.get_value(citer, 0) == connection:
@@ -445,29 +409,29 @@ class ConnectionsWidget(GladeWidget):
                     citer = model.iter_next(citer)
             iter = model.iter_next(iter)
 
-    def on_datasource_added(self, manager, datasource_info):
+    def on_datasource_added(self, manager, datasource):
         model = self.list_conn.get_model()
         iter = model.append(None)
-        model.set(iter, 0, datasource_info, 1, datasource_info.get_label())
+        model.set(iter, 0, datasource, 1, datasource.get_label())
 
-    def on_datasource_modified(self, manager, datasource_info):
+    def on_datasource_modified(self, manager, datasource):
         model = self.list_conn.get_model()
         if not model:
             return
         iter = model.get_iter_first()
         while iter:
-            if model.get_value(iter, 0) == datasource_info:
+            if model.get_value(iter, 0) == datasource:
                 model.set(iter,
-                          0, datasource_info,
-                          1, datasource_info.get_label())
+                          0, datasource,
+                          1, datasource.get_label())
                 return
             iter = model.iter_next(iter)
 
-    def on_datasource_deleted(self, manager, datasource_info):
+    def on_datasource_deleted(self, manager, datasource):
         model = self.list_conn.get_model()
         iter = model.get_iter_first()
         while iter:
-            if model.get_value(iter, 0) == datasource_info:
+            if model.get_value(iter, 0) == datasource:
                 model.remove(iter)
                 return
             iter = model.iter_next(iter)
@@ -478,30 +442,49 @@ class ConnectionsWidget(GladeWidget):
         is_ds = False
         if iter:
             obj = model.get_value(iter, 0)
-            is_ds = isinstance(obj, DatasourceInfo)
-            is_connection = isinstance(obj, DBConnection)
-        self.xml.get_widget("btn_disconnect").set_sensitive(is_connection)
-        self.xml.get_widget("btn_connect").set_sensitive(is_ds)
+            is_ds = isinstance(obj, Datasource)
+            is_connection = isinstance(obj, Connection)
+        self.builder.get_object('btn_disconnect').set_sensitive(is_connection)
+        self.builder.get_object('btn_connect').set_sensitive(is_ds)
+        btn_assign = self.builder.get_object('btn_assign')
+        editor = self.instance.get_active_editor()
+        btn_assign.set_sensitive(bool(is_connection and editor))
+
+    # --------------
+    # Public methods
+    # --------------
+
+    def run(self):
+        """Run the dialog."""
+        return self.dlg.run()
+
+    def destroy(self):
+        """Destroy the dialog."""
+        self.dlg.destroy()
 
     def refresh(self):
-        """Initializes the data model"""
+        """Refresh the data model"""
         model = self.list_conn.get_model()
-        for datasource_info in self.app.datasources.get_all():
+        for datasource in self.app.datasources.get_all():
             iter = model.append(None)
-            model.set(iter, 0, datasource_info, 1, datasource_info.get_label())
-            for conn in datasource_info.get_connections():
-                citer = model.append(iter)
-                model.set(citer, 0, conn, 1, conn.get_label())
-
-
-class ConnectionsDialog(GladeWidget):
-    """Dialog displaying connections"""
-
-    def __init__(self, app):
-        GladeWidget.__init__(self, app, "crunchyfrog", "connectionsdialog")
-
-    def _setup_widget(self):
-        self.connections = ConnectionsWidget(self.app, self.xml)
+            model.set(iter, 0, datasource,
+                      1, datasource.get_label(),
+                      2, False,
+                      3, pango.WEIGHT_BOLD,
+                      5, datasource.get_label())
+            citer = None
+            for conn in datasource.get_connections():
+                citer = model.append(None)
+                model.set(citer,
+                          0, conn,
+                          1, conn.get_label(short=True),
+                          2, False,
+                          5, conn.get_label())
+            if citer is None:
+                model.set_value(iter, 4, gtk.STOCK_DISCONNECT)
+            else:
+                model.set_value(iter, 4, gtk.STOCK_CONNECT)
+        self.list_conn.expand_all()
 
 
 class CustomImageMenuItem(gtk.ImageMenuItem):
