@@ -16,17 +16,23 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# $Id$
-
 """A grid view"""
 
 # NOTE:
 #     This module should have no cf dependencies!
 
+import mimetypes
+import tempfile
+
 import gtk
 import gobject
 import pango
-import gnomevfs
+
+try:
+    import gio
+    HAVE_GIO = True
+except ImportError:
+    HAVE_GIO = False
 
 import sys
 import os
@@ -40,8 +46,7 @@ class Grid(gtk.TreeView):
 
     The view component of the grid.
 
-    Signals
-    =======
+    :Signals:
 
         selection-changed
             ``def callback(grid, selected_cells, user_param1, ...)``
@@ -49,8 +54,7 @@ class Grid(gtk.TreeView):
             Emitted when selected cells have changed
 
 
-    Selection
-    =========
+    :Selection:
 
     This grid allows three different selections which exclude each
     other:
@@ -81,6 +85,7 @@ class Grid(gtk.TreeView):
         self.connect("button-press-event", self.on_button_pressed)
 
     def _setup_columns(self, rows):
+        sample_size = 50  # less samples, better performance :)
         renderer = gtk.CellRendererText()
         renderer.set_property("background", self.get_style().dark[gtk.STATE_ACTIVE].to_string())
         renderer.set_property("alignment", pango.ALIGN_CENTER)
@@ -98,6 +103,7 @@ class Grid(gtk.TreeView):
         self.append_column(col)
         offset_fg = len(self.description)*2
         offset_bg = len(self.description)*3
+        samples = set(rows[:sample_size])
         for i in range(len(self.description)):
             item = self.description[i]
             renderer = gtk.CellRendererText()
@@ -109,7 +115,7 @@ class Grid(gtk.TreeView):
                                      background_gdk=offset_bg+i)
             col.connect("clicked", self.on_column_header_clicked)
             col.set_resizable(True)
-            col.set_min_width(75)
+            col.set_fixed_width(self._get_best_column_width(i, samples))
             col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
             self.append_column(col)
         self.set_headers_clickable(True)
@@ -123,6 +129,26 @@ class Grid(gtk.TreeView):
             del old_model
         self.set_model(model)
 
+    def _get_best_column_width(self, colnum, samples):
+        label = '  %s  ' % self.description[colnum][0]
+        layout = self.create_pango_layout(label)
+        label_width = layout.get_pixel_size()[0]
+        lengths = set()
+        model = self.get_model()
+        for row in samples:
+            value = model._get_markup_for_value(row[colnum], markup=False)
+            lines = value.splitlines()
+            if lines:
+                value = lines[0]
+            del lines
+            layout = self.create_pango_layout('  %s  ' % value)
+            lengths.add(layout.get_pixel_size()[0])
+        if lengths:
+            max_length = max(lengths)
+        else:
+            max_length = 1
+        return min(max(max_length, label_width), label_width*3)
+
     def _get_popup_for_cell(self, row, col):
         col = self.get_model_index(col)
         model = self.get_model()
@@ -137,23 +163,29 @@ class Grid(gtk.TreeView):
                 sep = gtk.SeparatorMenuItem()
                 sep.show()
                 popup.append(sep)
-            if isinstance(data, buffer):
-                mime = gnomevfs.get_mime_type_for_data(data)
+            if isinstance(data, buffer) and HAVE_GIO:
+                mime = gio.content_type_guess(None, data)
                 item = gtk.MenuItem(_(u"Save as..."))
                 item.connect("activate", self.on_save_blob, data, mime)
                 item.show()
                 popup.append(item)
                 if mime:
-                    apps = gnomevfs.mime_get_all_applications(mime)
+                    apps = gio.app_info_get_all_for_type(mime)
                     if apps:
+                        default = gio.app_info_get_default_for_type(mime,
+                                                                    False)
+                        if default is not None and default in apps:
+                            apps.remove(default)
+                            apps.insert(0, default)  # make sure it's first
                         item = gtk.MenuItem(_(u"Open with..."))
                         smenu = gtk.Menu()
                         item.set_submenu(smenu)
                         item.show()
                         popup.append(item)
                         for app_info in apps:
-                            item = gtk.MenuItem(app_info[1])
-                            item.connect("activate", self.on_open_blob, data, app_info)
+                            item = gtk.MenuItem(app_info.get_name())
+                            item.connect("activate", self.on_open_blob,
+                                         data, app_info, mime)
                             item.show()
                             smenu.append(item)
             else:
@@ -185,6 +217,12 @@ class Grid(gtk.TreeView):
                     return
                 if not isinstance(data, buffer):
                     self.on_view_data(None, data)
+                elif isinstance(data, buffer) and HAVE_GIO:
+                    mime = gio.content_type_guess(None, data)
+                    default = gio.app_info_get_default_for_type(mime,
+                                                                False)
+                    if default is not None:
+                        self.on_open_blob(None, data, default, mime)
 
         elif event.button == 3:
             x = int(event.x)
@@ -232,13 +270,18 @@ class Grid(gtk.TreeView):
             self.select_column(xcolumn, selected)
         column.set_data("pressed", selected)
 
-    def on_open_blob(self, menuitem, data, app_info):
-        fname = os.path.expanduser("~/cf-blob.file")
-        cmd = "%s %s" % (app_info[2], fname)
-        f = open(fname, "w")
-        f.write(str(data))
+    def on_open_blob(self, menuitem, data, app_info, mime):
+        suffix = mimetypes.guess_extension(mime)
+        if suffix is None:
+            suffix = '.blob'
+        fp, fname = tempfile.mkstemp(suffix)
+        # no need to check for gio. The callback is only called iff gio
+        # is installed.
+        file_info = gio.File(fname)
+        f = open(fname, "wb")
+        f.write(data)
         f.close()
-        os.system(cmd)
+        app_info.launch([file_info])
 
     def on_save_blob(self, menuitem, data, mime):
         dlg = gtk.FileChooserDialog(_(u"Save as..."),
@@ -249,6 +292,9 @@ class Grid(gtk.TreeView):
         if mime:
             filter = gtk.FileFilter()
             filter.set_name(mime)
+            suffix = mimetypes.guess_extension(mime)
+            if suffix is not None:
+                filter.add_pattern('*.%s' % suffix)
             filter.add_mime_type(mime)
             dlg.add_filter(filter)
             dlg.set_filter(filter)
@@ -330,6 +376,9 @@ class Grid(gtk.TreeView):
         """Resets the grid"""
         old_model = self.get_model()
         if old_model:
+            self.unselect_cells()
+            self.unselect_rows()
+            self.unselect_columns()
             del old_model
         model = gtk.ListStore(int)
         self.set_model(model)
@@ -446,8 +495,8 @@ class Grid(gtk.TreeView):
                 DB-API2 like description
         """
         self.description = description
-        self._setup_columns(rows)
         self._setup_model(rows, description, coding_hint)
+        self._setup_columns(rows)
 
     def unselect_cells(self):
         """Unselects all cells"""
@@ -509,7 +558,18 @@ class GridModel(gtk.GenericTreeModel):
                 value = 'null'
         elif isinstance(value, buffer):
             if markup:
-                value = '<span foreground="%s">&lt;LOB&gt;</span>' % style.dark[gtk.STATE_PRELIGHT].to_string()
+                if HAVE_GIO:
+                    mime = gio.content_type_guess(None, value)
+                else:
+                    mime = None
+                if mime is None:
+                    mime = 'LOB'
+                else:
+                    mime = ('%s (%s)'  # e.g. "Python-Skript (text/x-python)"
+                            % (gio.content_type_get_description(mime),
+                               mime))
+                value = ('<span foreground="%s">&lt;%s&gt;</span>'
+                         % (style.dark[gtk.STATE_PRELIGHT].to_string(), mime))
             else:
                 value = str(buffer)
         else:
@@ -521,13 +581,15 @@ class GridModel(gtk.GenericTreeModel):
                 value = unicode(value)
             value = value.splitlines()
             if value:
-                if strip_length:
-                    value = value[0][:GRID_LABEL_MAX_LENGTH]
+                if strip_length and len(value[0]) > GRID_LABEL_MAX_LENGTH:
+                    value = ('%s <span foreground="%s">[...]</span>'
+                             % (gobject.markup_escape_text(
+                                    value[0][:GRID_LABEL_MAX_LENGTH]),
+                                style.dark[gtk.STATE_NORMAL].to_string()))
                 else:
-                    value = "\n".join(value)
+                    value = gobject.markup_escape_text("\n".join(value))
             else:
                 value = ""
-            value = gobject.markup_escape_text(value)
         return value
 
     def on_get_flags(self):
