@@ -16,8 +16,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# $Id$
-
 """User database
 
 The user database can be used by plugins to store some information.
@@ -26,20 +24,18 @@ It is also used by CrunchyFrog itself i.e. to store connection data.
 :USER_DB: Path to sqlite3 file
 """
 
+import logging
+import os.path
+import sqlite3
+import sys
+
 import gobject
 
-import os.path
-try:
-    import sqlite3 as sqlite
-except ImportError:
-    from pysqlite2 import dbapi2 as sqlite
+from cf import USER_CONFIG_DIR
+from cf.ui import dialogs
 
-from cf import USER_DIR
+USER_DB = os.path.join(USER_CONFIG_DIR, "user.db")
 
-USER_DB = os.path.join(USER_DIR, "user.db")
-
-import logging
-log = logging.getLogger("USERDB")
 
 class UserDB(gobject.GObject):
     """User database class.
@@ -47,8 +43,7 @@ class UserDB(gobject.GObject):
     An instance of this class is accessible through the ``userdb``
     attribute of an `CFApplication`_ instance.
 
-    Usage example
-    =============
+    :Usage example:
 
         .. sourcecode:: python
 
@@ -65,8 +60,7 @@ class UserDB(gobject.GObject):
             >>> app.userdb.drop_table("foo")
 
 
-    Instance attributes
-    ===================
+    :Instance attributes:
 
         :conn: Database connection (sqlite connection)
         :cursor: Database cursor (sqlite cursor)
@@ -100,9 +94,15 @@ class UserDB(gobject.GObject):
 
         This method creates the database if necessary.
         """
-        log.debug("Initializing user database")
+        logging.debug("Initializing user database: %s", self._user_db)
         create = not os.path.isfile(self._user_db)
-        self.conn = sqlite.connect(self._user_db)
+        try:
+            self.conn = sqlite3.connect(self._user_db)
+        except sqlite3.OperationalError, err:
+            logging.exception('Unable to open user database:')
+            dialogs.error(_(u'Unable to open user database'),
+                          '%s (%s)' % (str(err), self._user_db))
+            sys.exit(1)
         self.cursor = self.conn.cursor()
         if create:
             self.__create_userdb()
@@ -112,8 +112,11 @@ class UserDB(gobject.GObject):
         sql = "create table sy_table_version (id integer primary key, \
         tablename text, version text)"
         self.cursor.execute(sql)
-        from cf import datasources
-        datasources.check_userdb(self)
+        if not self.get_table_version("datasource"):
+            sql = ("create table datasource (id integer primary key, "
+                   "name text, description text, backend text, options text, "
+                   "last_accessed real, num_accessed integer)")
+            self.create_table("datasource", "0.1", sql)
 
     def get_cursor(self):
         """Returns a new DB-API compliant cursor.
@@ -139,8 +142,8 @@ class UserDB(gobject.GObject):
         where tablename=?"
         try:
             self.cursor.execute(sql, (table_name,))
-        except sqlite.OperationalError, e:
-            log.warning("sy_table_version not found: %s" % str(e))
+        except sqlite3.OperationalError, e:
+            logging.warning("sy_table_version not found: %s" % str(e))
             return None
         result = self.cursor.fetchone()
         if result:
@@ -162,12 +165,12 @@ class UserDB(gobject.GObject):
         :Returns: ``True`` if the table was successfully created, otherwise ``False``
         """
         if self.get_table_version(name):
-            log.error("Table '%s' already exists.", name)
+            logging.error("Table '%s' already exists.", name)
             return False
         try:
             self.cursor.execute(statement)
-        except sqlite.OperationalError, e:
-            log.error("Failed to create table '%s': %s", name, str(e))
+        except sqlite3.OperationalError, e:
+            logging.error("Failed to create table '%s': %s", name, str(e))
             return False
         sql = ("insert into sy_table_version (tablename, version) "
                "values (?,?)")
@@ -190,7 +193,7 @@ class UserDB(gobject.GObject):
         sql = "drop table %s" % name
         try:
             self.cursor.execute(sql)
-        except sqlite.OperationalError, e:
-            log.error("Failed to delete table: %s", str(e))
+        except sqlite3.OperationalError, e:
+            logging.error("Failed to delete table: %s", str(e))
             return False
         return True
